#!/usr/bin/python
# -*- coding: utf-8 -*-

import appier

from .job import JobAPI
from .node import NodeAPI

BASE_URL = "https://print.bemisc.com/api/"
""" The default base URL to be used when no other
base URL value is provided to the constructor """


class API(appier.API, JobAPI, NodeAPI):
    """
    Implementation of the Colony Print API specification
    for a simplified python client usage.
    """

    def __init__(self, *args, **kwargs):
        appier.API.__init__(self, *args, **kwargs)
        self.base_url = appier.conf("PRINT_BASE_URL", BASE_URL)
        self.key = appier.conf("PRINT_KEY", None)
        self.base_url = kwargs.get("base_url", self.base_url)
        self.key = appier.conf("key", self.key)

    def build(
        self,
        method,
        url,
        data=None,
        data_j=None,
        data_m=None,
        headers=None,
        params=None,
        mime=None,
        kwargs=None,
    ):
        auth = kwargs.pop("auth", True)
        if auth and self.key:
            headers["X-Secret-Key"] = self.key

    def ping(self):
        url = self.base_url + "ping"
        contents = self.get(url, auth=False)
        return contents


class Ping(dict):
    pass



from typing import TypedDict
from appier import API as BaseAPI

from .job import JobAPI
from .node import NodeAPI

BASE_URL: str = ...

class Ping(TypedDict):
    time: float

class API(BaseAPI, JobAPI, NodeAPI):
    def ping(self) -> Ping: ...



#!/usr/bin/python
# -*- coding: utf-8 -*-

import time

import appier


class JobAPI(object):

    def list_jobs(self, *args, **kwargs):
        url = self.base_url + "jobs"
        contents = self.get(url, **kwargs)
        return contents

    def get_job(self, id):
        url = self.base_url + "jobs/%s" % id
        contents = self.get(url)
        return contents

    def wait_job(self, id, sleep=0.5, iterations=20):
        for _ in range(iterations):
            time.sleep(sleep)
            try:
                job = self.get_job(id)
            except Exception:
                continue
            status = job.get("status", None)
            if not status == "finished":
                continue
            if not "result" in job:
                raise appier.OperationalError(message="No result found")
            return job["result"]

        raise appier.OperationalError(
            message="Maximum iterations reached, no output found"
        )


class JobInfo(dict):
    pass


class JobsResult(dict):
    pass


class PrintResult(dict):
    pass



#!/usr/bin/python
# -*- coding: utf-8 -*-

from . import base
from . import job
from . import node

from .base import BASE_URL, API, Ping
from .job import JobAPI, JobInfo, JobsResult, PrintResult
from .node import NodeAPI, Node



#!/usr/bin/python
# -*- coding: utf-8 -*-

from typing import Mapping, NotRequired, TypedDict

class JobResult(TypedDict):
    handler: str
    result: str
    receivers: NotRequired[list[str]]
    output_data: NotRequired[str]

class JobInfo(TypedDict):
    name: str
    node_id: str
    printer: str | None
    status: str | None
    result: NotRequired[JobResult]

JobsResult = Mapping[str, JobInfo]

class PrintResult(TypedDict):
    id: str
    name: str
    node_id: str
    printer: str | None
    data_length: int

class JobAPI:
    def list_jobs(self) -> JobsResult: ...
    def get_job(self, id: str) -> JobInfo: ...
    def wait_job(
        self, id: str, sleep: float = ..., iterations: int = ...
    ) -> JobResult: ...



#!/usr/bin/python
# -*- coding: utf-8 -*-

from typing import NotRequired, Sequence, TypedDict

from .job import JobsResult, PrintResult

class Node(TypedDict):
    name: str
    mode: str
    location: NotRequired[str]
    node_printer: NotRequired[str]
    engines: Sequence[str]

class NodeAPI:
    def list_nodes(self) -> Sequence[Node]: ...
    def jobs_node(self, id: str) -> JobsResult: ...
    def print_default_node(
        self,
        id: str,
        data: str | None = None,
        data_b64: str | None = None,
        name: str | None = None,
        type: str | None = None,
        format: str | None = None,
        options: dict | None = None,
    ) -> PrintResult: ...
    def print_hello_default_node(
        self,
        id: str,
        type: str | None = None,
        format: str | None = None,
        options: dict | None = None,
    ) -> PrintResult: ...
    def print_printer_node(
        self,
        id: str,
        printer: str,
        data: str | None = None,
        data_b64: str | None = None,
        name: str | None = None,
        type: str | None = None,
        format: str | None = None,
        options: dict | None = None,
    ) -> PrintResult: ...
    def print_hello_printer_node(
        self,
        id: str,
        printer: str,
        type: str | None = None,
        format: str | None = None,
        options: dict | None = None,
    ) -> PrintResult: ...



#!/usr/bin/python
# -*- coding: utf-8 -*-

import time
import base64
import pprint

import appier
import colony_print


def print(id, printer=None):
    api = colony_print.API()
    if printer:
        return api.print_hello_printer_node(id, printer)
    else:
        return api.print_hello_default_node(id)


def save(id, printer=None):
    api = colony_print.API()
    options = dict(send_email=False, save_output=True)
    if printer:
        result = api.print_hello_printer_node(id, printer, options=options)
    else:
        result = api.print_hello_default_node(id, options=options)

    job_id = result["id"]
    result = api.wait_job(job_id)

    if not "output_data" in result:
        raise appier.OperationalError(message="No output data in result")
    data_b64 = result["output_data"]
    data = base64.b64decode(data_b64)
    with open("output.pdf", "wb") as file:
        file.write(data)

    return result


if __name__ == "__main__":
    node = appier.conf("NODE", None)
    printer = appier.conf("PRINTER", None)
    save_pdf = appier.conf("SAVE_PDF", False, cast=bool)

    if not node:
        raise appier.OperationalError(message="No node defined")

    result = save(node, printer=printer) if save_pdf else print(node, printer=printer)
    pprint.pprint(result)
else:
    __path__ = []



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import setuptools

setuptools.setup(
    name="colony-print-api",
    version="0.1.0",
    author="Hive Solutions Lda.",
    author_email="development@hive.pt",
    description="Colony Print API Client",
    license="Apache License, Version 2.0",
    keywords="colony-print api",
    url="http://colony-print-api.hive.pt",
    zip_safe=False,
    packages=["colony_print", "colony_print.scripts"],
    test_suite="colony_print.test",
    package_dir={"": os.path.normpath("src")},
    package_data={"colony_print": ["*.pyi"]},
    install_requires=["appier"],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Topic :: Utilities",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.0",
        "Programming Language :: Python :: 3.1",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
    ],
    long_description=open(os.path.join(os.path.dirname(__file__), "README.md"), "rb")
    .read()
    .decode("utf-8"),
    long_description_content_type="text/markdown",
)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import json


class NodeAPI(object):

    def list_nodes(self, *args, **kwargs):
        url = self.base_url + "nodes"
        contents = self.get(url, **kwargs)
        return contents

    def jobs_node(self, id):
        url = self.base_url + "nodes/%s/jobs" % id
        contents = self.get(url)
        return contents

    def print_default_node(
        self,
        id,
        data=None,
        data_b64=None,
        name=None,
        type=None,
        format=None,
        options=None,
    ):
        url = self.base_url + "nodes/%s/print" % id
        contents = self.post(
            url,
            params=dict(
                data=data,
                data_b64=data_b64,
                name=name,
                type=type,
                format=format,
                options=json.dumps(options) if options else None,
            ),
        )
        return contents

    def print_hello_default_node(
        self,
        id,
        type=None,
        format=None,
        options=None,
    ):
        url = self.base_url + "nodes/%s/print_hello" % id
        contents = self.post(
            url,
            params=dict(
                type=type,
                format=format,
                options=json.dumps(options) if options else None,
            ),
        )
        return contents

    def print_printer_node(
        self,
        id,
        printer,
        data=None,
        data_b64=None,
        name=None,
        type=None,
        format=None,
        options=None,
    ):
        url = self.base_url + "nodes/%s/printers/print" % id
        contents = self.post(
            url,
            params=dict(
                printer=printer,
                data=data,
                data_b64=data_b64,
                name=name,
                type=type,
                format=format,
                options=json.dumps(options) if options else None,
            ),
        )
        return contents

    def print_hello_printer_node(
        self, id, printer, type=None, format=None, options=None
    ):
        url = self.base_url + "nodes/%s/printers/print_hello" % id
        contents = self.post(
            url,
            params=dict(
                printer=printer,
                type=type,
                format=format,
                options=json.dumps(options) if options else None,
            ),
        )
        return contents


class Node(dict):
    pass
